package edu.chl.hajo.monopoly.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

import edu.chl.hajo.monopoly.constants.Piece;
import edu.chl.hajo.monopoly.core.space.Property;
import edu.chl.hajo.monopoly.core.space.Street;
import edu.chl.hajo.monopoly.event.EventBus;
import edu.chl.hajo.monopoly.event.GameEvents;
import edu.chl.hajo.monopoly.event.StateChanges;
import edu.chl.hajo.monopoly.util.Config;
import edu.chl.hajo.monopoly.util.PlayerData;

/**
 * A player in the game
 * 
 * @author hajo
 * 
 */
public class Player implements IPlayer {

	// The value part of Player
	private final String name;
	private final Piece piece;
	private boolean inJail;
	private int balance;
	private final Collection<Property> properties;

	// Used in model
	private int position = 0;
	private final EventBus eb = EventBus.getInstance();
	private final IRules rules;
	private final Board2 board;
	private final Dices dices;

	public Player(PlayerData data, IRules rules, 
			Board2 board, Dices dices) {
		super();
		this.name = data.name;
		this.piece = data.piece;
		this.balance = data.balance;
		this.inJail = data.inJail;
		if (data.properties == null) {
			this.properties = new ArrayList<Property>();
		} else {
			this.properties = data.properties;
		}
		this.rules = rules;
		this.board = board;
		this.dices = dices;
	}

	// ---------- IPlayer -----------------
	@Override
	public PlayerData data() {
		return new PlayerData(name, piece, balance, inJail,
				Collections.unmodifiableCollection(properties));
	}

	@Override
	public boolean withdraw(int amount) {
		if (balance >= amount) {
			balance -= amount;
			eb.send(this, StateChanges.PLAYER_WITHDRAW, null, balance);
			return true;
		} else {
			// Calculate player total
			// if total < amount bankrupcy
			// ...else...
			eb.send(this, GameEvents.PLAYER_MUST_SELL, null, amount);
			return false;
		}
	}

	@Override
	public void addIncome(int income) {
		balance += income;
		eb.send(this, StateChanges.PLAYER_ADD_INCOME, null, balance);
	}

	@Override
	public void buyBuilding(Street street) {
		if (!rules.buyBuildingEnabled(street, this)) {
			return;
		}
		Building b = Bank.getBuilding();
		if (balance >= b.getPrice()) {
			street.addBuilding(b);
			withdraw(b.getPrice());
		} else {
			eb.broadcast(GameEvents.NO_MONEY, null, null);
		}
	}

	@Override
	public void sellBuilding(Street street) {

		if (!rules.sellBuildingEnabled(street, this)) {
			return;
		}

		Building b = street.removeBuilding();
		// Bank pays half
		addIncome(b.getPrice() / Config.getResellPriceReduction());
	}

	@Override
	public void buyProperty(Property property) {
		// Blocked by GUI but with this we can test!!
		if (!rules.buyEnabled(property, this)) {
			return;
		}
		if (balance >= property.getPrice()) {
			withdraw(property.getPrice());
			property.setOwner(this);
			properties.add(property);
		} else {
			eb.broadcast(GameEvents.NO_MONEY, null, null);
		}
	}

	@Override
	public void sellProperty(Property property) {
		if (!rules.sellPropertyEnabled(property, this)) {
			return;
		}
		properties.add(property);
		property.setOwner(null);
		addIncome(property.getPrice());
	}

	@Override
	public void move() {
		int steps = dices.roll();
		move(steps);
	}

	@Override
	public void goToJail() {
		board.remove(this);
		position = 10;
		inJail = true;
		board.put(this);
	}

	@Override
	public boolean isInJail() {
		return inJail;
	}

	@Override
	public void freeFromJail() {
		if (withdraw(1000)) { // TODO
			inJail = false;
		} else {
			eb.send(this, StateChanges.PLAYER_IN_JAIL, null, false);
		}
	}

	// ------- Accessors ----------------

	public int getPosition() {
		return position;
	}

	// ------- Testing ---------------
	// Also called from move()
	public void move(int steps) {
		board.remove(this);
		int pos = position;
		if (!isInJail()) {
			int nSpaces = board.getSize();
			pos = pos + steps;
			if (pos >= nSpaces) {
				pos %= nSpaces;
				addIncome(Config.getPassGoIncome());
			}
			position = pos;
		} else {
			if (dices.areEqual()) {
				position = pos;
				inJail = false;
			}
		}
		board.put(this);
	}
}
